﻿local addon, ns = ...



-- PreviewRelicFrame
-- ArtifactRelicForgePreviewRelicMixin
--[==[

	function ArtifactRelicForgePreviewRelicMixin:OnEnter()
		local relicItemLink = C_ArtifactRelicForgeUI.GetPreviewRelicItemLink();
		if ( relicItemLink ) then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
			GameTooltip:SetHyperlink(relicItemLink);
		else
			local type, itemID, itemLink = GetCursorInfo();
			if type ~= "item" or not IsArtifactRelicItem(itemID) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
				GameTooltip:SetText(RELIC_FORGE_PREVIEW_RELIC_TOOLTIP_TITLE, HIGHLIGHT_FONT_COLOR:GetRGB());
				GameTooltip:AddLine(RELIC_FORGE_PREVIEW_RELIC_TOOLTIP, nil, nil, nil, true);
				GameTooltip:Show();
			end
		end
	end
	
	function ArtifactRelicTalentButtonMixin:GetChosenChild()
		local talents = C_ArtifactRelicForgeUI.GetSocketedRelicTalents(self:GetParent().relicSlot);
		if ( not talents ) then
			return nil;
		end

		local selfIndex = self:GetID();
		for buttonIndex, layoutInfo in ipairs(TALENTS_LAYOUT) do
			for linkIndex, parentButtonIndex in ipairs(layoutInfo.links) do
				if ( parentButtonIndex == selfIndex and talents[buttonIndex].isChosen ) then
					return self:GetParent().Talents[buttonIndex];
				end
			end
		end
		return nil;
	end
]==]
-- local PREVIEW_RELIC_SLOT = 4;

-- function ArtifactRelicForgePreviewRelicMixin:OnDragStart()
--	C_ArtifactRelicForgeUI.PickUpPreviewRelic();
--end

-- PickupContainerItem(container, slot)

--[==[
self.powerID = powerID;
self.spellID = powerInfo.spellID;
self.currentRank = powerInfo.currentRank;
self.bonusRanks = powerInfo.bonusRanks;
self.maxRank = powerInfo.maxRank;
self.isStart = powerInfo.isStart;
self.isGoldMedal = powerInfo.isGoldMedal;
self.isFinal = powerInfo.isFinal;
self.tier = powerInfo.tier;
self.textureKit = textureKit;
self.linearIndex = powerInfo.linearIndex;
self.numMaxRankBonusFromTier = powerInfo.numMaxRankBonusFromTier	
]==]

local ILVL_TAG = 'ilvl'
local PATTERN_RELIC_ILVL = "(%d+%,?%.?%d*) "..RELIC_ITEM_LEVEL_INCREASE..'$'


local hidegametooltip = CreateFrame("Frame")
hidegametooltip:Hide()
local gametooltip = CreateFrame("GameTooltip", addon.."ParseGameToolTip", nil, "GameTooltipTemplate");
gametooltip:SetOwner( hidegametooltip,"ANCHOR_NONE");
gametooltip:SetScript('OnTooltipAddMoney', function()end)
gametooltip:SetScript('OnTooltipCleared', function()end)
gametooltip:SetScript('OnTooltipSetDefaultAnchor',function()end)

function ns.GetRelicInfo(relicID)
	gametooltip:ClearLines()
	
	local totalilvl = 0
	local powerBonus
	local itemID = C_ArtifactUI.GetArtifactInfo() or C_ArtifactUI.GetEquippedArtifactInfo()
	
	if relicID == 4 then
		local relicItemLink = C_ArtifactRelicForgeUI.GetPreviewRelicItemLink();
		
		if relicItemLink then
			gametooltip:SetHyperlink(relicItemLink);
			
			totalilvl = totalilvl + 5
		else
			return totalilvl
		end
	else	
		gametooltip:SetSocketedRelic(relicID);
	end
	
	for i=1, 15 do        
		local left = _G[gametooltip:GetName().."TextLeft"..i]:GetText()
		
		if left then
		
			local ilvl = string.match(left, PATTERN_RELIC_ILVL)
			
			
			if ilvl then
				totalilvl = totalilvl + ( tonumber(ilvl) or 0 )
			end
			
			if not powerBonus and itemID and CrucibleWeightDB.artifacts[itemID] then
				for powerID, spellID in pairs(CrucibleWeightDB.artifacts[itemID]) do
					if string.match(left, (GetSpellInfo(spellID))) then
						powerBonus = spellID
						break
					end
				end
			end				
		end
	end
	
	return totalilvl, powerBonus
end

local unknownIcon = "\124TInterface\\Icons\\Inv_misc_questionmark:16\124t "..UNKNOWN

local function SpellString(spellID)
	if not spellID then 
		return unknownIcon	
	end
	
	local name, _, icon = GetSpellInfo(spellID)
			
	if not name then
		return unknownIcon	
	end
	
	return "\124T"..icon..":16\124t "..name			
end

local RelicForgeSpells = {
	-- Void Traits
	[252091] = 'Master of Shadows',
	[252191] = 'Murderous Intent',
	[252875] = 'Shadowbind',
	[252888] = 'Chaotic Darkness',
	[252906] = 'Torment the Weak',
	[252922] = 'Dark Sorrows',
	
	-- Light Traits
	[252088] = 'Light Speed',
	[252207] = 'Refractive Shell',
	[252799] = 'Shocklight',
	[253070] = 'Secure in the Light',
	[253093] = 'Infusion of Light',
	[253111] = "Light's Embrace",
}

function ns:ScanRelics()	
	for i=1, 3 do
		local talents = C_ArtifactRelicForgeUI.GetSocketedRelicTalents(i);
		if ( talents ) then
			print('Relic slot', i)
			for k,v in pairs(talents) do
				print('  Index:', k, 'powerID:', v.powerID, 'tier:', v.tier, 'link:', C_ArtifactUI.GetPowerHyperlink(v.powerID), C_ArtifactUI.GetPowerInfo(v.powerID))
			end
		else
			print('No relic in slot', i)
		end
	end			
end

function ns:ScanPreviewRelic()
	local talents = C_ArtifactRelicForgeUI.GetPreviewRelicTalents();
	if ( talents ) then
		print('Relic slot', 4)
		for k,v in pairs(talents) do
			print('  Index:', k, 'powerID:', v.powerID, 'tier:', v.tier, 'link:', C_ArtifactUI.GetPowerHyperlink(v.powerID), C_ArtifactUI.GetPowerInfo(v.powerID))
		end
	else
		print('No relic in slot', 4)
	end
end

--ns.relicSlotValue = {}
function ns:BuildRelicValue(index)
	wipe(ns.relicSlotValue)
	
	local talents = index == 4 and C_ArtifactRelicForgeUI.GetPreviewRelicTalents() or C_ArtifactRelicForgeUI.GetSocketedRelicTalents(index);
	if ( talents ) then
	--	print('Relic slot', index)
		for k,v in pairs(talents) do
			local spellID = C_ArtifactUI.GetPowerInfo(v.powerID).spellID
	--		print('  Index:', k, 'powerID:', v.powerID, 'tier:', v.tier, 'link:', C_ArtifactUI.GetPowerHyperlink(v.powerID), C_ArtifactUI.GetPowerInfo(v.powerID).spellID )
			
			if spellID and ns.GetValue(spellID) then				
				ns.relicSlotValue[k] = ns.GetValue(spellID) and tonumber(ns.GetValue(spellID))
			end
		end
	end
	
	for a=1, #ns.talentElement do
		if ns.relicSlotValue[ns.talentElement[a].id] then
			local color
			local value = ns.relicSlotValue[ns.talentElement[a].id] or 0
			if ns.talentElement[a].id == 2 or ns.talentElement[a].id == 3 then
				color = ns.GetNumberPosition(ns.relicSlotValue[2], ns.relicSlotValue[3])	
			else
				color = ns.GetNumberPosition(ns.relicSlotValue[4], ns.relicSlotValue[5], ns.relicSlotValue[6])	
			end
		
			color = ns.GetColorPreset(color[value] or 0)
			
			ns.talentElement[a].text:SetText(ns.relicSlotValue[ns.talentElement[a].id] or '??')
			ns.talentElement[a].text:SetTextColor(color[1],color[2],color[3],1)
			ns.talentElement[a].texture:SetColorTexture(color[1],color[2],color[3],0.4)
		else
			ns.talentElement[a].text:SetText('??')
			ns.talentElement[a].text:SetTextColor(ns.GetColorPreset(0)[1],ns.GetColorPreset(0)[2],ns.GetColorPreset(0)[3],1)
			ns.talentElement[a].texture:SetColorTexture(ns.GetColorPreset(0)[1],ns.GetColorPreset(0)[2],ns.GetColorPreset(0)[3],0.4 )
		end
	end
end

do
	local sortfunc = function(x,y)
		return x>y
	end
		
	function ns.GetNumberPosition(...)		
		local t = {}
		
		for i=1, select('#', ...) do
			local number = select(i, ...)
			local addit = true
			
			for a=1, #t do				
				if t[a] == number then		
					addit = true
				end
			end
			
			t[#t+1] = number
		end
		
		table.sort(t, sortfunc)
		
		local t1 = {}
		
		for k,v in pairs(t) do
			t1[v]=k
		end
		
		return t1
	end
	
	local colors = {
		[1] = { 0, 1, 0 },
		[2] = { 1, 1, 0 },
		[3] = { 1, 0, 0 },
	}
	function ns.GetColorPreset(index)		
		return colors[index] or { 0.5, 0.5, 0.5 }
	end
	
end

ns.TotalRelicValue = {}
for i=1, 4 do
	ns.TotalRelicValue[i] = { all = {}, a1 = 0, a2 = 0, a3 = 0, a4 = 0 }
end

--[==[
		1
	2		3
4		5		6
]==]

local relicMap = {
	[2] = { 
		req = 1,
		[3] = false
	},
	[3] = {
		req = 1,
		[2] = false,
	},
	[4] = { 
		req = 4,
		[5] = false,
		[6] = false,		
		[2] = true,
		[3] = false,
	},
	[5] = { 
		req = 3,
		[4] = false,
		[6] = false,		
		[2] = true,
		[3] = true,
	},
	[6] = { 
		req = 4,
		[4] = false,
		[5] = false,
		[3] = true,
		[2] = false,
	},
}

function ns:IsReachable(data, index)

	local t = relicMap[index]
	
	local status = 0
	
	if t then
		for k, v  in pairs(data) do
			if t[k] == v.isChosen then
				status = status + 1
			end
			
			if status == t.req then
				return true --'st1-true-'..status
			elseif status == t.req-1 then
				return true --'st2-true-'..status
			end
		end
	end
	
	return false --'st3-false-'..status
end

function ns:BuildTotalRelicValue(index)
	ns.TotalRelicValue[index] = { all = {}, a1 = 0, a2 = 0, a3 = 0, a4 = 0 }
	
	local talents = index == 4 and C_ArtifactRelicForgeUI.GetPreviewRelicTalents() or C_ArtifactRelicForgeUI.GetSocketedRelicTalents(index);
	
	if ( talents ) then
	--	print('Relic slot', index)
		for k,v in pairs(talents) do
		
			-- canChoose
			-- isChosen
			
			if k == 1 then
			
			else
				local spellID = C_ArtifactUI.GetPowerInfo(v.powerID).spellID
				local isValid = v.canChoose or v.isChosen
				
		--		print('  Index:', k, 'powerID:', v.powerID, 'tier:', v.tier, 'Valid:', ns:IsReachable(talents, k) )
				
				if spellID and ns.GetValue(spellID) and ns:IsReachable(talents, k) then				
					ns.TotalRelicValue[index].all[k] = ns.GetValue(spellID) and tonumber(ns.GetValue(spellID)) or 0
				else
					ns.TotalRelicValue[index].all[k] = 0
				end
			end
		end
	end
	
	local totalitemlevel, powerID = ns.GetRelicInfo(index)
	
	local itemLevel = ( totalitemlevel and ns.GetValue(ILVL_TAG) ) and totalitemlevel*ns.GetValue(ILVL_TAG) or 0
	local powerLevel = ( powerID and ns.GetValue(powerID) ) and ns.GetValue(powerID) or 0
	local totalPower = itemLevel + powerLevel
	
	ns.TotalRelicValue[index].a1 = ( ns.TotalRelicValue[index].all[2] or 0 ) + ( ns.TotalRelicValue[index].all[4] or 0 ) + totalPower
	ns.TotalRelicValue[index].a2 = ( ns.TotalRelicValue[index].all[2] or 0 ) + ( ns.TotalRelicValue[index].all[5] or 0 ) + totalPower
	
	ns.TotalRelicValue[index].a3 = ( ns.TotalRelicValue[index].all[3] or 0 ) + ( ns.TotalRelicValue[index].all[5] or 0 ) + totalPower
	ns.TotalRelicValue[index].a4 = ( ns.TotalRelicValue[index].all[3] or 0 ) + ( ns.TotalRelicValue[index].all[6] or 0 ) + totalPower
	
--	local relicItemLink = C_ArtifactRelicForgeUI.GetPreviewRelicItemLink();
	
	local a1, a2, a3, a4 = ns.TotalRelicValue[index].a1, ns.TotalRelicValue[index].a2, ns.TotalRelicValue[index].a3, ns.TotalRelicValue[index].a4
	
	local m1 = math.max(ns.TotalRelicValue[1].a1, ns.TotalRelicValue[1].a2, ns.TotalRelicValue[1].a3, ns.TotalRelicValue[1].a4)
	local m2 = math.max(ns.TotalRelicValue[2].a1, ns.TotalRelicValue[2].a2, ns.TotalRelicValue[2].a3, ns.TotalRelicValue[2].a4)
	local m3 = math.max(ns.TotalRelicValue[3].a1, ns.TotalRelicValue[3].a2, ns.TotalRelicValue[3].a3, ns.TotalRelicValue[3].a4)
	local m4 = math.max(ns.TotalRelicValue[4].a1, ns.TotalRelicValue[4].a2, ns.TotalRelicValue[4].a3, ns.TotalRelicValue[4].a4)
	
	local currentMax = (index==1 and m1 or index==2 and m2 or index==3 and m3 or index==4 and m4)
	
	local color = ns.GetNumberPosition(m1, m2, m3, m4)	
	color = ns.GetColorPreset(color[currentMax] or 0)
	
	ns.gui.RelicItemLevels[index].text:SetText( currentMax )
	ns.gui.RelicItemLevels[index].text:SetTextColor(color[1], color[2], color[3],1)
	ns.gui.RelicItemLevels[index].texture:SetColorTexture(color[1], color[2], color[3], 0.4)
	
--	print("ItemLevel", ns.GetRelicItemLevel(index)*(ns.GetValue(ILVL_TAG) or 0) , 'a1=', ns.TotalRelicValue[index].a1, 'a2=', ns.TotalRelicValue[index].a2, 'a3=', ns.TotalRelicValue[index].a3, 'a4=', ns.TotalRelicValue[index].a4)
end

local relicBagCache = {}
local relicParseList = {}
local ignorePreviewLookUp = false
local startload = true

function ns:LookUpRelics()	
	print('ns:LookUpRelics')
	for bagID = 0, 4 do
		for invID = 1, GetContainerNumSlots(bagID) do
			local itemID = GetContainerItemID(bagID, invID)
			
		--	print('LookUpRelics', bagID, invID, itemID, itemID and IsArtifactRelicItem(itemID))
			
			if itemID and IsArtifactRelicItem(itemID) then
				local index = 0
				
				local scan = true
				
				for i=1, #relicBagCache do 
					local d = relicBagCache[i]
					if d[1] == bagID and d[2] == invID then
						scan = false
					end
				end
				
				if scan then
					relicParseList[#relicParseList+1] = { bagID, invID }
					
				--	print('relicParseList', bagID, invID)
				--	C_ArtifactRelicForgeUI.ClearPreviewRelic();
					PickupContainerItem(bagID, invID)
					
					local canSet, bindWarning = C_ArtifactRelicForgeUI.CanSetPreviewRelicFromCursor();
					if ( canSet ) then
						if ( bindWarning ) then
							
						else
							C_ArtifactRelicForgeUI.SetPreviewRelicFromCursor();
							local isAttuned, canAttune = C_ArtifactRelicForgeUI.GetPreviewRelicAttuneInfo();
							if ( canAttune ) then
								C_ArtifactRelicForgeUI.AttunePreviewRelic();
							end
						end
					else
						ResetCursor()
					end
				end
			end
		end
	end
	
	startload = false
	ignorePreviewLookUp = true
	C_ArtifactRelicForgeUI.ClearPreviewRelic();
	ResetCursor()
	ignorePreviewLookUp = false
end

ns.relicSlotValue = {}

local loader = CreateFrame('Frame')
loader:RegisterEvent('ADDON_LOADED')
loader:RegisterEvent("ARTIFACT_RELIC_FORGE_UPDATE");
loader:RegisterEvent("ARTIFACT_RELIC_FORGE_CLOSE");
loader:RegisterEvent("ARTIFACT_RELIC_FORGE_PREVIEW_RELIC_CHANGED");
loader:SetScript('OnEvent', function(self, event, name)
	
	if name == addon then
		print(addon..' Loaded')
		
		local profileName = UnitName('player')..' - '..GetRealmName()
		
		CrucibleWeightDB = CrucibleWeightDB or {}
		CrucibleWeightDB.artifacts = CrucibleWeightDB.artifacts or {}
		CrucibleWeightDB.chars = CrucibleWeightDB.chars or {}
		
		if not CrucibleWeightDB.chars[profileName] then
			CrucibleWeightDB.chars[profileName] = {}
			CrucibleWeightDB.chars[profileName].artifactWeight = {}		
		end
		
		ns.profile = CrucibleWeightDB.chars[profileName]
	elseif name == 'Blizzard_ArtifactUI' then
	
		ns.artifactName = _G.ArtifactRelicForgeFrame:CreateFontString()
		ns.artifactName:SetFont(STANDARD_TEXT_FONT, 12, 'OUTLINE')
		ns.artifactName:SetPoint('TOP', _G.ArtifactRelicForgeFrame, 'TOP', 0, -30)
		ns.artifactName:SetText('')
		
		
		
		ns.mainFrame:SetParent(_G.ArtifactRelicForgeFrame)
		ns.mainFrame:SetPoint('TOPLEFT', _G.ArtifactRelicForgeFrame, 'TOPRIGHT', 20, 0)
	
		
			
		ns.talentElement[#ns.talentElement+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.Talent2,2)
		ns.talentElement[#ns.talentElement+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.Talent3,3)
		ns.talentElement[#ns.talentElement+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.Talent4,4)
		ns.talentElement[#ns.talentElement+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.Talent5,5)
		ns.talentElement[#ns.talentElement+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.Talent6,6)
		
		ns.gui.RelicItemLevels[#ns.gui.RelicItemLevels+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.TitleContainer.RelicSlot1,1)
		ns.gui.RelicItemLevels[#ns.gui.RelicItemLevels]:SetPoint('TOP', ArtifactRelicForgeFrame.TitleContainer.RelicSlot1, 'BOTTOM', 0, 5)
		
		ns.gui.RelicItemLevels[#ns.gui.RelicItemLevels+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.TitleContainer.RelicSlot2,2)
		ns.gui.RelicItemLevels[#ns.gui.RelicItemLevels]:SetPoint('TOP', ArtifactRelicForgeFrame.TitleContainer.RelicSlot2, 'BOTTOM', 0, 5)
		
		ns.gui.RelicItemLevels[#ns.gui.RelicItemLevels+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.TitleContainer.RelicSlot3,3)
		ns.gui.RelicItemLevels[#ns.gui.RelicItemLevels]:SetPoint('TOP', ArtifactRelicForgeFrame.TitleContainer.RelicSlot3, 'BOTTOM', 0, 5)
		
		ns.gui.RelicItemLevels[#ns.gui.RelicItemLevels+1] = ns.CreaetTalentElement(ArtifactRelicForgeFrame.PreviewRelicFrame,4)
		
		hooksecurefunc(ArtifactRelicForgeFrame, 'SetRelicSlot', function(self, relicID)			
		--	print('SetRelicSlot', relicID)
			
			ns.lastSetRelicSlot = relicID
			
			ns:BuildRelicValue(ns.lastSetRelicSlot)
			
			for i=1, 4 do
				ns:BuildTotalRelicValue(i)
			end
			
			-- ns.talentElement
		end)
		
	--	print('Blizzard_ArtifactUI Loaded')	
		
	--	self:RegisterEvent("ARTIFACT_RELIC_FORGE_UPDATE");
	--	self:RegisterEvent("ARTIFACT_RELIC_FORGE_CLOSE");
	--	self:RegisterEvent("ARTIFACT_RELIC_FORGE_PREVIEW_RELIC_CHANGED");
		
	--	startload = true
		
	--	ns:ScanRelics()	
	--	ns:LookUpRelics()	
	elseif event == 'ARTIFACT_RELIC_FORGE_UPDATE' then		
	--	print(event)
		
	--	print(C_ArtifactUI.GetEquippedArtifactInfo())
		
		local itemID, altItemID, name, icon, xp, pointsSpent, quality = C_ArtifactUI.GetEquippedArtifactInfo()
		
		ns.artifactName:SetText(name)
		ns.artifactName:SetTextColor(GetItemQualityColor(quality))
		
		ns.mainFrame:Show()
		
		ns.UpdateArtifactGUIElements()
		ns.UpdateValues()
		
		if not startload then return end
		startload = true
		
	--	ns:ScanRelics()	
	--	ns:LookUpRelics()	
	elseif event == 'ARTIFACT_RELIC_FORGE_CLOSE' then
	--	print(event)
		
		ns.mainFrame:Hide()
		
		startload = true
		
	elseif event == 'ARTIFACT_RELIC_FORGE_PREVIEW_RELIC_CHANGED' then
	
		ns.UpdateValues()
		
		if ignorePreviewLookUp then return end
		if not startload then return end
		
		
	--	ns:ScanPreviewRelic()
	end
end)

function ns.ComfirmAndSave(editBox, spellID, value)
	local itemID = C_ArtifactUI.GetEquippedArtifactInfo()
	ns.profile[itemID] = ns.profile[itemID] or {}
	
	if value == '' then
		ns.profile[itemID][spellID] = nil
		editBox:SetText('')
	else
		local number = tonumber(value)
		
		if number then
			ns.profile[itemID][spellID] = number
			editBox:SetText(number)
		else
			ns.profile[itemID][spellID] = nil
			editBox:SetText('')
		end
	end
	
	ns.UpdateValues()
end

function ns.GetValue(spellID)
	local itemID = C_ArtifactUI.GetEquippedArtifactInfo()
	
	if ns.profile[itemID] and ns.profile[itemID][spellID] then		
		return ns.profile[itemID][spellID]
	end
	
	return false
end

function ns.UpdateValues()
	for k,v in pairs(ns.gui.Elements) do		
		for i, element in pairs(v) do
			element:SetText(ns.GetValue(element.spellID) or '')
		end
	end
	
	ns.gui.ItemLevel:SetText(ns.GetValue(ILVL_TAG) or '')
	
	ns:BuildRelicValue(ns.lastSetRelicSlot or 1)
	
	for i=1, 4 do
		ns:BuildTotalRelicValue(i)
	end
end

-- Artifact parses
do
	-- Artifact Parser
	local function PrepareForScan()
		local ArtifactFrame = _G.ArtifactFrame
		
		if not ArtifactFrame or not ArtifactFrame:IsShown() then
	--		print('PrepareForScan', 'Success')
			
			_G.UIParent:UnregisterEvent("ARTIFACT_UPDATE")
			if ArtifactFrame then
				ArtifactFrame:UnregisterEvent("ARTIFACT_UPDATE")
			end
		end
	end

	local function RestoreStateAfterScan()
		local ArtifactFrame = _G.ArtifactFrame
			
		if not ArtifactFrame or not ArtifactFrame:IsShown() then
			C_ArtifactUI.Clear()
			
	--		print('RestoreStateAfterScan', 'Success')
			
			if ArtifactFrame then
				ArtifactFrame:RegisterEvent("ARTIFACT_UPDATE")
			end
			_G.UIParent:RegisterEvent("ARTIFACT_UPDATE")
		end
	end

	local lastUpdate = -1
	local init = true
	local initUpdate = true

	local function GetArtifactPercInfo()
		initUpdate = true
		
		if not HasArtifactEquipped() then		
			lastUpdate = -1			
			return false
		end
		
		if init or lastUpdate < GetTime() then
			lastUpdate = GetTime() + 2
			init = false

			PrepareForScan()
			SocketInventoryItem(INVSLOT_MAINHAND)
			
			local powers = C_ArtifactUI.GetPowers();
			local itemID = C_ArtifactUI.GetEquippedArtifactInfo();

			if powers then
				if itemID and 
					not CrucibleWeightDB.artifacts[itemID] then
						CrucibleWeightDB.artifacts[itemID] = {}
				end
				
				local total = 0
				
				for i, powerID in ipairs(powers) do
					local powerInfo = C_ArtifactUI.GetPowerInfo(powerID);
					
					local spellID, cost, currentRank, maxRank, bonusRanks
					
					if powerInfo and type(powerInfo) == 'table' then
						spellID = powerInfo.spellID;
						maxRank = powerInfo.maxRank;
					else
						spellID, cost, currentRank, maxRank, bonusRanks = C_ArtifactUI.GetPowerInfo(powerID);	
					end
					
					if maxRank and maxRank > 1 and maxRank < 10 then
						total = total + 1
						CrucibleWeightDB.artifacts[itemID][powerID] = spellID
						
					--	print(total, maxRank, GetSpellLink(spellID))
					else
					--	print('Ignore', maxRank, GetSpellLink(spellID))
					end
				end
			end
			
			RestoreStateAfterScan()
		end
	end
	
	
	local eventframe = CreateFrame("Frame")
	eventframe:SetScript("OnEvent", function(self, event, unit)
		if event == 'ARTIFACT_UPDATE' then
			if not unit then
				if initUpdate then
					initUpdate = false
					C_Timer.After(0.5, GetArtifactPercInfo)
				end
			end
		else
			if initUpdate then
				initUpdate = false
				C_Timer.After(0.5, GetArtifactPercInfo)
			end
		end
	end)
	eventframe:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED")
	eventframe:RegisterEvent("PLAYER_TALENT_UPDATE")
	eventframe:RegisterEvent("PLAYER_LEVEL_UP")
	eventframe:RegisterEvent("PLAYER_LOGIN")
	eventframe:RegisterEvent("ARTIFACT_CLOSE")
	eventframe:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")
end

if GetLocale() == 'enUS' or GetLocale() == 'enGB' then

else
	hooksecurefunc(GameTooltip, 'SetArtifactPowerByID', function(self, powerID)
		local powerData = C_ArtifactUI.GetPowerInfo(powerID)
		if powerData and RelicForgeSpells[powerData.spellID] then
	--		print(powerID, powerData.spellID, RelicForgeSpells[powerData.spellID])
			
			GameTooltip:AddLine('  ')
			GameTooltip:AddLine('Eng - '..RelicForgeSpells[powerData.spellID], GetItemQualityColor(6))
		end
	end)
end

-- GUI

local mainFrame = CreateFrame('Frame', nil, _G.UIParent)
mainFrame:SetSize(300, 560)
mainFrame:Hide()
mainFrame:EnableMouse(true)

mainFrame.border = mainFrame:CreateTexture()
mainFrame.border:SetAllPoints()
mainFrame.border:SetColorTexture(0,0,0,1)
mainFrame.border:SetDrawLayer('BORDER')

mainFrame.background = mainFrame:CreateTexture()
mainFrame.background:SetPoint('TOPLEFT', mainFrame, 'TOPLEFT', 1, -1)
mainFrame.background:SetPoint('BOTTOMRIGHT', mainFrame, 'BOTTOMRIGHT', -1, 1)
mainFrame.background:SetColorTexture(0.1,0.1,0.1,1)
mainFrame.background:SetDrawLayer('ARTWORK')

mainFrame.export = CreateFrame("Button", nil, mainFrame, "UIPanelButtonTemplate")
	mainFrame.export:SetFrameLevel(mainFrame:GetFrameLevel()+1)
	mainFrame.export:SetSize(60,20)
	mainFrame.export:SetPoint("BOTTOM", mainFrame, "BOTTOM", -45, 5)
	mainFrame.export:Show()

	mainFrame.export:SetScript("OnClick", function(self)
		PlaySound(SOUNDKIT and SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON or "igMainMenuOptionCheckBoxOn")
		
		ns:Popup('export')
	end)

	mainFrame.export.text = mainFrame.export:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
	mainFrame.export.text:SetPoint("CENTER", mainFrame.export, "CENTER", 0 , 0)
	mainFrame.export.text:SetTextColor(1, 0.8, 0)
	mainFrame.export.text:SetText("Export")
	mainFrame.export.text:SetJustifyH("CENTER")
	mainFrame.export.text:SetWordWrap(false)
	

mainFrame.import = CreateFrame("Button", nil, mainFrame, "UIPanelButtonTemplate")
	mainFrame.import:SetFrameLevel(mainFrame:GetFrameLevel()+1)
	mainFrame.import:SetSize(60,20)
	mainFrame.import:SetPoint("BOTTOM", mainFrame, "BOTTOM", 45, 5)
	mainFrame.import:Show()

	mainFrame.import:SetScript("OnClick", function(self)
		PlaySound(SOUNDKIT and SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON or "igMainMenuOptionCheckBoxOn")
		
		ns:Popup('import')
	end)

	mainFrame.import.text = mainFrame.import:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
	mainFrame.import.text:SetPoint("CENTER", mainFrame.import, "CENTER", 0 , 0)
	mainFrame.import.text:SetTextColor(1, 0.8, 0)
	mainFrame.import.text:SetText("import")
	mainFrame.import.text:SetJustifyH("CENTER")
	mainFrame.import.text:SetWordWrap(false)


ns.mainFrame = mainFrame


local RelicForgeSpells = {
	-- Void Traits
	[252091] = 'Master of Shadows',
	[252191] = 'Murderous Intent',
	[252875] = 'Shadowbind',
	[252888] = 'Chaotic Darkness',
	[252906] = 'Torment the Weak',
	[252922] = 'Dark Sorrows',
	
	-- Light Traits
	[252088] = 'Light Speed',
	[252207] = 'Refractive Shell',
	[252799] = 'Shocklight',
	[253070] = 'Secure in the Light',
	[253093] = 'Infusion of Light',
	[253111] = "Light's Embrace",
}

local relicPercsPosition = {
	['VOID']  = {252091,252191,252875,252888,252906,252922},
	['LIGHT'] = {252088,252207,252799,253070,253093,253111},
}

function ns.CreateElement()
	
	local f = CreateFrame("EditBox", nil, mainFrame)
	f:SetFontObject('ChatFontNormal')
	f:SetFrameLevel(mainFrame:GetFrameLevel() + 1)
	f:SetAutoFocus(false)
	f:SetWidth(160)
	f:SetHeight(20)
	f:SetBackdrop({
			bgFile = [[Interface\Buttons\WHITE8x8]],
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			edgeSize = 1,
			insets = {top = 0, left = 0, bottom = 0, right = 0},
				})
	f:SetBackdropColor(0, 0, 0, 0.2)
	f:SetBackdropBorderColor(0,0,0,1)
		
	f:SetScript("OnEscapePressed", function(self)
		self:ClearFocus()
		self.ok:Hide()
		self:SetText(ns.GetValue(self.spellID) or '')
	end)
	f:SetScript('OnEnterPressed', function(self)
		self:ClearFocus()
		self.ok:Hide()
		
		ns.ComfirmAndSave(self, self.spellID, self:GetText())
	end)
	
	local text = f:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
	text:SetPoint("BOTTOMLEFT", f, "TOPLEFT", -15 , 0)
	text:SetJustifyH("LEFT")
	text:SetWordWrap(false)
	
	local okbttm = CreateFrame("Button", nil, f, "UIPanelButtonTemplate")
	okbttm:SetFrameLevel(f:GetFrameLevel()+1)
	okbttm:SetSize(30,20)
	okbttm:SetPoint("RIGHT", f, "RIGHT", 0, 0)
	okbttm:Hide()

	okbttm:SetScript("OnClick", function(self)
		PlaySound(SOUNDKIT and SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON or "igMainMenuOptionCheckBoxOn")
		self:GetParent():ClearFocus()
		self:Hide()
		ns.ComfirmAndSave( self:GetParent(), self:GetParent().spellID, self:GetParent():GetText())
	end)
	
	okbttm.text = okbttm:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
	okbttm.text:SetPoint("CENTER", okbttm, "CENTER", 0 , 0)
	okbttm.text:SetTextColor(1, 0.8, 0)
	okbttm.text:SetText("OK")
	okbttm.text:SetJustifyH("CENTER")
	okbttm.text:SetWordWrap(false)
	
	f:SetScript("OnTextChanged", function(self, userInput)	
		if userInput then
			self.ok:Show()
		end
	end)
	
	
	f.mouseover = CreateFrame("Frame", nil, f)
	f.mouseover:SetFrameLevel(f:GetFrameLevel()+2)
	f.mouseover:SetSize(1,1)
	f.mouseover:SetPoint("TOPLEFT", text, "TOPLEFT", -3, 3)
	f.mouseover:SetPoint("BOTTOMRIGHT", text, "BOTTOMRIGHT", 3, -3)
	f.mouseover:SetScript("OnEnter", function(self)	
		if self:GetParent().spellID == ILVL_TAG then		
			return
		end
		
		GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
		GameTooltip:SetHyperlink('spell:'..self:GetParent().spellID)
		
		if GetLocale() == 'enUS' or GetLocale() == 'enGB' then

		else
			if self:GetParent().spellID and RelicForgeSpells[self:GetParent().spellID] then
				GameTooltip:AddLine('  ')
				GameTooltip:AddLine('Eng - '..RelicForgeSpells[self:GetParent().spellID], GetItemQualityColor(6))
			end
		end
		
		GameTooltip:Show()
		
		ns.HighLightBySpellID(self:GetParent().spellID)
		
	--	print('T', 'spell:'..self:GetParent().spellID)
	end)
	f.mouseover:SetScript("OnLeave", function(self)		
		GameTooltip:Hide()
		ns.HighLightHide()
	end)
	
	f.ok = okbttm
	f.text = text
	
	return f
end

ns.gui = {}
ns.gui.Elements = {}

local VoidText = mainFrame:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
	VoidText:SetPoint("CENTER", mainFrame, "TOP", -62 , -20)
	VoidText:SetJustifyH("CENTER")
	VoidText:SetSize(100, 20)
	VoidText:SetText('Void')

VoidText.background = mainFrame:CreateTexture()
	VoidText.background:SetSize(120, 240)
	VoidText.background:SetPoint('TOP', VoidText, 'BOTTOM', 0, -1)
	VoidText.background:SetColorTexture(80/255, 83/255, 150/255, 0.4)
	VoidText.background:SetDrawLayer('ARTWORK', 1)


local LightText = mainFrame:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
	LightText:SetPoint("CENTER", mainFrame, "TOP", 62 , -20)
	LightText:SetJustifyH("CENTER")
	LightText:SetSize(100, 20)	
	LightText:SetText('Light')
	
LightText.background = mainFrame:CreateTexture()
	LightText.background:SetSize(120, 240)
	LightText.background:SetPoint('TOP', LightText, 'BOTTOM', 0, -1)
	LightText.background:SetColorTexture(1,0.7,0.5,0.4)
	LightText.background:SetDrawLayer('ARTWORK', 1)
	
for side in pairs(relicPercsPosition) do
	ns.gui.Elements[side] = {}
	
	for i=1, #relicPercsPosition[side] do
		local spellID = relicPercsPosition[side][i]
		
		ns.gui.Elements[side][i] = ns.CreateElement()
		ns.gui.Elements[side][i].text:SetWidth(110)
		if i ==  1 then
			ns.gui.Elements[side][i]:SetPoint('TOP', side == 'VOID' and VoidText or LightText, 'BOTTOM', 0, -20)
		else
			ns.gui.Elements[side][i]:SetPoint('TOP', ns.gui.Elements[side][i-1], 'BOTTOM', 0, -16)
		end
		
		ns.gui.Elements[side][i].spellID = spellID
		ns.gui.Elements[side][i].text:SetText(SpellString(spellID))
		ns.gui.Elements[side][i]:SetWidth(70)
		ns.gui.Elements[side][i]:Show()
	end
end


local ArtifactText = mainFrame:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
	ArtifactText:SetPoint("CENTER", mainFrame, "TOP", 0 , -280)
	ArtifactText:SetJustifyH("CENTER")
	ArtifactText:SetSize(100, 20)	
	ArtifactText:SetText('Artifact')
	
ArtifactText.background = mainFrame:CreateTexture()
	ArtifactText.background:SetSize(220, 220)
	ArtifactText.background:SetPoint('TOP', ArtifactText, 'BOTTOM', 0, -1)
		local r,g,b = GetItemQualityColor(6)
	ArtifactText.background:SetColorTexture(r,g,b,0.4)
	ArtifactText.background:SetDrawLayer('ARTWORK', 1)
	
ns.gui.Elements['ARTIFACT'] = {}

for i=1, 10 do
	
	ns.gui.Elements['ARTIFACT'][i] = ns.CreateElement()
	ns.gui.Elements['ARTIFACT'][i]:SetWidth(70)
	ns.gui.Elements['ARTIFACT'][i].text:SetWidth(105)
	
	if i==1 then
		ns.gui.Elements['ARTIFACT'][i]:SetPoint('TOP', ArtifactText, 'BOTTOM', -55, -20)
	elseif i == 6 then
		ns.gui.Elements['ARTIFACT'][i]:SetPoint('TOP', ArtifactText, 'BOTTOM', 55, -20)
	else
		ns.gui.Elements['ARTIFACT'][i]:SetPoint('TOP', ns.gui.Elements['ARTIFACT'][i-1], 'BOTTOM', 0, -15)
	end
end
	
ns.gui.ItemLevel = ns.CreateElement()
ns.gui.ItemLevel:SetWidth(70)
ns.gui.ItemLevel.spellID = ILVL_TAG
ns.gui.ItemLevel.text:SetText('+1 iLvL')
ns.gui.ItemLevel.text:SetWidth(0)
ns.gui.ItemLevel.text:ClearAllPoints()
ns.gui.ItemLevel.text:SetPoint('BOTTOM', ns.gui.ItemLevel, 'TOP', 0, 2)
ns.gui.ItemLevel:SetPoint('TOP', ArtifactText, 'BOTTOM', 0, -195)
	
function ns.UpdateArtifactGUIElements()
	local itemID = C_ArtifactUI.GetEquippedArtifactInfo();
	
	local index = 1
	if CrucibleWeightDB.artifacts[itemID] then
		for powerID, spellID in pairs( CrucibleWeightDB.artifacts[itemID] ) do
			
			ns.gui.Elements['ARTIFACT'][index].text:SetText(SpellString(spellID))
			ns.gui.Elements['ARTIFACT'][index].spellID = spellID
			
			index = index + 1
		end
	end
end

ns.talentElement = {}
function ns.CreaetTalentElement(parent, index)
	local f = CreateFrame('Frame', nil,  parent)
	f:SetSize(30, 20)
	f:SetPoint('TOP', parent, 'BOTTOM', 0, -10)
	f.id = index
	
	f.texture = f:CreateTexture()
	f.texture:SetAllPoints()
	f.texture:SetColorTexture(r,g,b,0.4)
	f.texture:SetDrawLayer('OVERLAY', 1)
	
	f.text = f:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
	f.text:SetPoint("CENTER", f, "CENTER", 0 , 0)
	f.text:SetJustifyH("CENTER")
	f.text:SetText('0')
	
	f.highlight = f:CreateTexture()
	f.highlight:SetSize(70,70)
	f.highlight:SetPoint('CENTER',parent,'CENTER')
	f.highlight:SetColorTexture(1,1,1,0.2)
	f.highlight:SetDrawLayer('OVERLAY', 1)
	f.highlight:Hide()
	
	return f
end


ns.gui.RelicItemLevels = {}

function ns.HighLightBySpellID(spellID2)
	ns.HighLightHide()
	
	for i=1, #ns.talentElement do
		local button = ns.talentElement[i]:GetParent()
		local spellID = C_ArtifactUI.GetPowerInfo(button.powerID).spellID
		
		if spellID == spellID2 then
			ns.talentElement[i].highlight:Show()
		end
	end	
end

function ns.HighLightHide()
	for i=1, #ns.talentElement do		
		ns.talentElement[i].highlight:Hide()
	end	
end

do
	local width = 400
	
	local editBox
	local hide
	local frame = CreateFrame("Frame", nil, UIParent)

	frame:SetBackdrop({
			bgFile = [[Interface\Buttons\WHITE8x8]],
			edgeFile = [[Interface\Buttons\WHITE8x8]],
			edgeSize = 1,
			insets = {left = 0, right = 0, top = 0, bottom =0}
		})
	frame:SetBackdropColor(0 , 0 , 0 , 0.7) --цвет фона
	frame:SetBackdropBorderColor(0 , 0 , 0 , 1) --цвет фона
	
	frame:SetSize(width, 40)
	frame:SetPoint("CENTER", UIParent, "CENTER")
	frame:SetFrameStrata("DIALOG")
	frame:Hide()

	local import = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
		import:SetFrameLevel(frame:GetFrameLevel()+1)
		import:SetSize(60,20)
		import:Show()

		import:SetScript("OnClick", function(self)
			PlaySound(SOUNDKIT and SOUNDKIT.IG_MAINMENU_OPTION_CHECKBOX_ON or "igMainMenuOptionCheckBoxOn")
			ns.GetImportString(editBox:GetText())
			frame:Hide()
		end)

		import.text = import:CreateFontString(nil, 'OVERLAY', "GameFontNormalSmall")
		import.text:SetPoint("CENTER", import, "CENTER", 0 , 0)
		import.text:SetTextColor(1, 0.8, 0)
		import.text:SetText("import")
		import.text:SetJustifyH("CENTER")
		import.text:SetWordWrap(false)
		
	editBox = CreateFrame("EditBox", nil, frame)
	editBox:SetFontObject(ChatFontNormal)
	editBox:SetSize(width-120, 40)
	editBox:SetPoint("LEFT", frame, "LEFT", 10, 0)
	
	hide = function(f) f:GetParent():Hide() end
	
	editBox:SetScript("OnEscapePressed", hide)
	editBox:SetScript('OnTextChanged', function(self, userInput)
		if userInput then
			if self.lastText then				
				self:SetText(self.lastText)
			else
				local text = self:GetText()

				if text:find('cruweight') then
					import:Show()
				else
					import:Hide()
				end
			end
		end
	end)
	--[==[
	local fsting = editBox:CreateFontString()
	fsting:SetFontObject(ChatFontNormal)
	fsting:SetFont(STANDARD_TEXT_FONT, 12, 'NONE')
	fsting:SetPoint("LEFT", editBox, "RIGHT", 5, 0)
	fsting:SetText('|cFF606060Ctrl+C - '..'COPY'..'\nCtrl+V - '..'PASTE'..'|r')
	fsting:SetJustifyH('LEFT')
	]==]
	local close_ = CreateFrame("Button", nil, frame)
	close_:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
	close_:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
	close_:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight", "ADD")
	close_:SetSize(32, 32)
	close_:SetPoint("RIGHT", frame, "RIGHT", -5, 0)
	close_:SetScript("OnClick", hide)
	
	import:SetPoint("RIGHT", close_, "LEFT", -5, 0)
	
	--[[ End popup creation ]]--
	
	-- Avoiding StaticPopup taints by making our own popup, rather that adding to the StaticPopup list
	function ns:Popup(type)
	
		if type == 'export' then
			local text = ns.BuildExportString()
			
			editBox.lastText = text
			editBox:SetText(text)
			editBox:HighlightText(0)
			import:Hide()
		else
			editBox.lastText = nil
			editBox:SetText('')
			import:Hide()
		end
		
		editBox:GetParent():Show()
	end
end


function ns.BuildExportString()
	local itemID = C_ArtifactUI.GetEquippedArtifactInfo()
	
	local text = nil
	
	if ns.profile[itemID] then		
		text = 'cruweight^'..itemID
		
		for arg, value in pairs(ns.profile[itemID]) do
			
			text = text..'^'..arg..'^'..value
		end
		
		text = text..'^end'
	end
	
	return text
end

function ns.GetImportString(text)
	local info = { strsplit('^', text) }
	
	local tag = info[1]
	local artifactID = info[2] and tonumber(info[2])
	
	ns.profile[artifactID] = {}
	
	for i=3, #info, 2 do
		if info[i] and info[i+1] then
			if info[i] == 'end' then
			
			elseif info[i] == ILVL_TAG then
				local arg = info[i]
				local value = tonumber(info[i+1]) 
				
				ns.profile[artifactID][arg] = value
			else
				local arg = tonumber(info[i]) 
				local value = tonumber(info[i+1]) 
				
				ns.profile[artifactID][arg] = value
			end		
		end
	end
	
	ns.UpdateValues()
end